/***************************************************************************
 *   Copyright (C) 2010 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "omwifi.h"
#include <QApplication>
#include <QDebug>

OMWiFi::OMWiFi(QWidget *parent)
 : QWidget(parent)
{
	IconBasePath = "/usr/share/pixmaps/openmoko-panel-plugin-qt/";
	activeIconFile = "wifi_1.png";
	inactiveIconFile = "wifi_0.png";


	activeIcon = QIcon(IconBasePath + activeIconFile);
	inactiveIcon = QIcon(IconBasePath + inactiveIconFile);

	MainLayout = new QVBoxLayout();
	ActivateLayout = new QHBoxLayout();

	StatusPanelName = QObject::tr("status:");
	
	Configuration = new OMConfig("openmoko-panel-plugin-qt-wifi", 0 );

	InfoBox = new OMInfoBox(QObject::tr("Device"), this);
	InfoBox->addPanel(StatusPanelName, "");
	MainLayout->addWidget( InfoBox );

	ActivateButton = new QPushButton(QObject::tr("turn wireless on"));
	ActivateLayout->addWidget(ActivateButton);
	connect(ActivateButton,SIGNAL(clicked()),this,SLOT(ActivateButtonclicked()));

	DeactivateButton = new QPushButton(QObject::tr("turn wireless off"));
	ActivateLayout->addWidget(DeactivateButton);
	connect(DeactivateButton,SIGNAL(clicked()),this,SLOT(DeactivateButtonclicked()));

	MainLayout->addLayout(ActivateLayout);


	DialogButtons = new OMDialogButtons(OMFlags::QuitButton | OMFlags::CloseButton | OMFlags::ConfigButton, this);
	MainLayout->addWidget( DialogButtons );
	connect(DialogButtons, SIGNAL(clicked(QFlags< OMFlags :: Flag >)), this, SLOT(DialogButtonClicked(QFlags< OMFlags :: Flag >)));


	setLayout( MainLayout );

	SysTrayIcon = new QSystemTrayIcon( inactiveIcon, this );
	connect(SysTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	SysTrayIcon->show();


	systemBus = new QDBusConnection( QDBusConnection::connectToBus( QDBusConnection::SystemBus, "system" ) );
	connectToUsageInterface();
	if (systemBus->connect("org.freesmartphone.ousaged", "/org/freesmartphone/Usage", "org.freesmartphone.Usage","ResourceChanged", this, SLOT(ResourceChanged(QString,bool)) )) qDebug() << "Signal ResourceChanged connected";
}


OMWiFi::~OMWiFi()
{
}

void OMWiFi::iconActivated(QSystemTrayIcon::ActivationReason)
{
    showMaximized ();
}


void OMWiFi::DeactivateButtonclicked()
{
	Configuration->runPreDisableScript();
	if (UsageInterface->isValid()) UsageInterface->call("ReleaseResource","WiFi");
}


void OMWiFi::ActivateButtonclicked()
{
	Configuration->runPreEnableScript();
	if (UsageInterface->isValid()) UsageInterface->call("RequestResource","WiFi");

}

void OMWiFi::connectToUsageInterface()
{
	UsageInterface = new OMUsageInterface( this );
	
	if (UsageInterface->isValid()) {
		
		QDBusReply <bool> reply = UsageInterface->call("GetResourceState","WiFi");
		ResourceChanged("WiFi", reply.value());
		
		}
	else qDebug() << "could not connect to usage interface";
}

void OMWiFi::ResourceChanged(QString name, bool state)
{
    if (name == "WiFi") {

	if (state) {
		SysTrayIcon->setIcon( activeIcon );	
		InfoBox->changePanelValue( StatusPanelName, QObject::tr("active") );
		Configuration->runPostEnableScript();
		}

	else {

		SysTrayIcon->setIcon( inactiveIcon );		
		InfoBox->changePanelValue( StatusPanelName, QObject::tr("inactive") );
		Configuration->runPostDisableScript();


		}

	}

}


void OMWiFi::DialogButtonClicked(QFlags<OMFlags::Flag> flags)
{
    if ( flags.testFlag(OMFlags::QuitButton) ) qApp->quit();
    if ( flags.testFlag(OMFlags::CloseButton) ) hide();
    if ( flags.testFlag(OMFlags::ConfigButton) ) Configuration->showMaximized();

}